@@ -102,7 +102,7 @@ class ALS11012RSoXSLoader(FileLoader):
                     print(f'Loading dark for {md["EXPOSURE"]} from {file}')
                     exptime = md['EXPOSURE']
                     self.darks[exptime] = img
-    def loadSingleImage(self,filepath,coords=None,return_q=False):
+    def loadSingleImage(self,filepath,coords=None,return_q=False,**kwargs):
         '''
         THIS IS A HELPER FUNCTION, mostly - should not be called directly unless you know what you are doing
 
@@ -110,6 +110,8 @@ class ALS11012RSoXSLoader(FileLoader):
         Load a single image from filepath and return a single-image, raw xarray, performing dark subtraction if so configured.
 
         '''
+        if len(kwargs.keys())>0:
+            warnings.warn(f'Loader does not support features for args: {kwargs.keys()}',stacklevel=2)
         input_image = fits.open(filepath)
         headerdict =  self.normalizeMetadata(dict(zip(input_image[0].header.keys(),input_image[0].header.values())))
         img = input_image[2].data
@@ -192,4 +194,4 @@ class ALS11012RSoXSLoader(FileLoader):
         headerdict['det_th'] = round(headerdict['CCD Theta'],2)
         headerdict.update(self.constant_md)
         return headerdict
-        
\ No newline at end of file
+        
