@@ -8,11 +8,18 @@ import warnings
 import json
 #from pyFAI import azimuthalIntegrator
 import numpy as np
+import pandas as pd
+from tqdm.auto import tqdm
+import scipy.ndimage
+import asyncio
+import time
+
+
 
 try:
     os.environ["TILED_SITE_PROFILES"] = '/nsls2/software/etc/tiled/profiles'
     from tiled.client import from_profile
-    from databroker.queries import RawMongo
+    from databroker.queries import RawMongo, Key, FullText, Contains,Regex
 except:
     print('Imports failed.  Are you running on a machine with proper libraries for databroker, tiled, etc.?')
     
@@ -85,8 +92,77 @@ class SST1RSoXSDB:
         '''
         q = RawMongo(**kwargs)
         return self.c.search(q)
-    
-    def loadRun(self,run,dims=None,coords={}):
+ 
+    def summarize_run(self,proposal=None,saf=None,user=None,institution=None,project=None,sample=None,plan=None):
+            '''
+            Returns a Pandas dataframe with a summary of runs matching a set of search criteria.
+
+            Args:
+                proposal, saf, user, institution (str or None): if str, adds an exact match search on the appropriate parameter to the set
+                project,sample,plan (str or None): if str, adds a regex match search on the appropriate parameter to the set.
+                    example: project='*Liquid*' matches 'Liquid','Liquids','Liquid-RSoXS')
+
+            Returns:
+                pd.Dataframe containing the results of the search.
+            '''
+            catalog = self.c
+            if proposal is not None:
+                catalog = catalog.search(Key('proposal_id')==proposal)
+            if saf is not None:
+                catalog = catalog.search(Key('saf_id')==saf)
+            if user is not None:
+                catalog = catalog.search(Key('user_name')==user)
+            if institution is not None:
+                catalog = catalog.search(Key('institution')==institution)
+            if project is not None:
+                catalog = catalog.search(Regex("project_name",project))
+            if sample is not None:
+                catalog = catalog.search(Regex('sample_name',sample))
+            if plan is not None:
+                catalog = catalog.search(Regex('plan_name',plan))
+            cat = catalog
+            #print(cat)
+            #print('#    scan_id        sample_id           plan_name')
+            scan_ids = []
+            sample_ids = []
+            plan_names = []
+            start_times = []
+            npts = []
+            uids = []
+            for num,entry in tqdm((enumerate(cat)),total=len(cat)):
+                doc = catalog[entry].start
+                scan_ids.append(doc["scan_id"])
+                sample_ids.append(doc["sample_id"])
+                plan_names.append(doc["plan_name"])
+                uids.append(doc["uid"])
+                try:
+                    npts.append(catalog[entry].stop['num_events']['primary'])
+                except KeyError:
+                    npts.append(0)
+                start_times.append(doc["time"])
+                #do_list_append(catalog[entry],scan_ids,sample_ids,plan_names,uids,npts,start_times)
+                #print(f'{num}  {cat[entry].start["scan_id"]}  {cat[entry].start["sample_id"]} {cat[entry].start["plan_name"]}')
+            return pd.DataFrame(list(zip(scan_ids,sample_ids,plan_names,npts,uids,start_times)),
+                       columns =['scan_id', 'sample_id','plan_name','npts','uid','time'])
+
+    def background(f):
+        def wrapped(*args, **kwargs):
+            return asyncio.get_event_loop().run_in_executor(None, f, *args, **kwargs)
+
+        return wrapped
+    @background
+    def do_list_append(run,scan_ids,sample_ids,plan_names,uids,npts,start_times):
+        doc = run.start
+        scan_ids.append(doc["scan_id"])
+        sample_ids.append(doc["sample_id"])
+        plan_names.append(doc["plan_name"])
+        uids.append(doc["uid"])
+        try:
+            npts.append(run.stop['num_events']['primary'])
+        except KeyError:
+            npts.append(0)
+        start_times.append(doc["time"])
+    def loadRun(self,run,dims=None,coords={},return_dataset=False):
         '''
         Loads a run entry from a catalog result into a raw xarray.
 
@@ -94,15 +170,16 @@ class SST1RSoXSDB:
             run (DataBroker result): a single run from BlueSky
             dims (list): list of dimensions you'd like in the resulting xarray.  See list of allowed dimensions in documentation.  If not set or None, tries to auto-hint the dims from the RSoXS plan_name.
             coords (dict): user-supplied dimensions, see syntax examples in documentation.
-
+            return_dataset (bool,default False): return both the data and the monitors as a xr.dataset.  If false (default), just returns the data.
         Returns:
             raw (xarray): raw xarray containing your scan in PyHyper-compliant format
 
         '''
         md = self.loadMd(run)
+        monitors = self.loadMonitors(run)
         if 'NEXAFS' in md['start']['plan_name']:
             raise NotImplementedError(f"Scan {md['start']['scan_id']} is a {md['start']['plan_name']} NEXAFS scan.  NEXAFS loading is not yet supported.")
-        elif ('full' in md['start']['plan_name'] or 'short' in md['start']['plan_name']) and dims is None:
+        elif ('full' in md['start']['plan_name'] or 'short' in md['start']['plan_name'] or 'custom_rsoxs_scan' in md['start']['plan_name']) and dims is None:
             dims = ['energy']
         elif 'spiralsearch' in md['start']['plan_name'] and dims is None:
             dims = ['sam_x','sam_y']
@@ -129,9 +206,7 @@ class SST1RSoXSDB:
                 return img + pedestal - darks[int(img.dark_id.values)]
             data = data.groupby('time').map(subtract_dark,darks=dark,pedestal=self.dark_pedestal)
 
-        if self.corr_mode != 'none':
-            warnings.warn('corrections other than none are not supported at the moment',stacklevel=2)
-
+      
 
         dims_to_join = []
         dim_names_to_join = []
@@ -160,9 +235,18 @@ class SST1RSoXSDB:
         
         #this is needed for holoviews compatibility, hopefully does not break other features.
         retxr = retxr.assign_coords({'pix_x':np.arange(0,len(retxr.pix_x)),'pix_y':np.arange(0,len(retxr.pix_y))})
-        
+        monitors = monitors.rename({'time':'system'}).reset_index('system').assign_coords(system=index).drop('system_')
         retxr.attrs.update(md)
+          
+        #now do corrections:
+        frozen_attrs = retxr.attrs
+        if self.corr_mode == 'i0':
+            retxr = retxr / monitors['RSoXS Au Mesh Current']
+        elif self.corr_mode != 'none':
+            warnings.warn('corrections other than none are not supported at the moment',stacklevel=2)
         
+        retxr.attrs.update(frozen_attrs)
+
         # deal with the edge case where the LAST energy of a run is repeated... this may need modification to make it correct (did the energies shift when this happened??)
         try:
             if retxr.system[-1] == retxr.system[-2]:
@@ -170,13 +254,45 @@ class SST1RSoXSDB:
         except IndexError:
             pass
         
+        if return_dataset:
+            #retxr = (index,monitors,retxr)
+            monitors.attrs.update(retxr.attrs)
+            retxr = monitors.merge(retxr)
+            
+            
         return retxr
 
 
     def peekAtMd(self,run):
         return self.loadMd(run)
 
+    def loadMonitors(self,entry,integrate_onto_images=True,n_thinning_iters=20):
+        '''
+        Load the monitor streams for entry.
+        Args:
+           entry (Bluesky document): run to extract monitors from
+           integrate_onto_images (bool, default True): return integral of monitors while shutter was open for images.  if false, returns raw data.
+           n_thinning_iters (int, default 20): how many iterations of binary thinning to use to exclude shutter edges.
+        
+        '''
+        monitors = None
+        for stream_name in list(entry.keys()):
+            if 'monitor' in stream_name:
+                if monitors is None:
+                    monitors = entry[stream_name].data.read()
+                else:
+                    monitors = xr.merge((monitors,entry[stream_name].data.read()))
+        monitors = monitors.ffill('time').bfill('time')
+        if integrate_onto_images:
+            monitors['RSoXS Shutter Toggle_thinned'] = monitors['RSoXS Shutter Toggle']
+            monitors['RSoXS Shutter Toggle_thinned'].values = scipy.ndimage.binary_erosion(monitors['RSoXS Shutter Toggle'].values,iterations=n_thinning_iters,border_value=0)
+            monitors = monitors.where(monitors['RSoXS Shutter Toggle_thinned']>0).dropna('time')
+            monitors = monitors.groupby_bins('time',
+np.insert(entry.primary.data['time'].values,0,0)).mean().rename_dims({'time_bins':'time'})
+            monitors = monitors.assign_coords({'time':entry.primary.data['time']}).reset_coords('time_bins',drop=True)
 
+        return monitors
+    
     def loadMd(self,run):
         '''
         return a dict of metadata entries from the databroker run xarray
@@ -242,7 +358,9 @@ class SST1RSoXSDB:
             'sam_x':'RSoXS Sample Outboard-Inboard',
             'sam_y':'RSoXS Sample Up-Down',
             'sam_z':'RSoXS Sample Downstream-Upstream',
+            'sam_th':'RSoXS Sample Rotation',
             'polarization':'en_polarization_setpoint',
+            'energy':'en_energy_setpoint',
             'exposure':'RSoXS Shutter Opening Time (ms)' #md['detector']+'_cam_acquire_time'
         }
 
@@ -262,7 +380,7 @@ class SST1RSoXSDB:
         md['epoch'] = md['meas_time'].timestamp()
                     
         md['wavelength'] = 1.239842e-6 / md['energy']
-        #md['sampleid'] = scan_id@todo this should be easy
+        md['sampleid'] = start['scan_id']
 
         md['dist'] = md['sdd'] / 1000
 
